<?php

namespace Drupal\indieauth\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Returns responses for Indieauth routes.
 */
class IndieauthController extends ControllerBase {

  const PROVIDER = 'indieauth';

  /**
   * The immutable config object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The controller constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('indieauth.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Builds the response.
   */
  public function build() {
    $request = \Drupal::requestStack()->getCurrentRequest();
    $authorization_code = $request->get('code');
    $me = $request->get('me');
    $state = $request->get('state');

    // Verify needed parameters are available.
    if (!$authorization_code
      || !$me
      || !$state) {
      return new JsonResponse([
        'error' => 'invalid_request',
        'error_description' => 'One or all of the parameters authorization_code, me ore state are missing.',
      ], 400);
    }

    // Verify state token.
    $cid = implode(':', ['indieauth', $me, $state]);
    $authorization = \Drupal::cache()->get($cid)->data;
    if ($state !== $authorization['state']) {
      return new JsonResponse([
        'error' => 'invalid_request',
        'error_description' => 'The state token is invalid.',
      ], 400);
    }

    // Verify authorization code.
    try {
      $response = \Drupal::httpClient()->post(
        \Drupal::config('indieauth.settings')->get('authorization_endpoint'), [
          'headers' => [
            'Accept' => 'application/json',
          ],
          'form_params' => [
            'code' => $authorization_code,
            'redirect_uri' => Url::fromRoute('indieauth.endpoint')->setAbsolute()->toString(),
            'client_id' => Url::fromRoute('<front>')->setAbsolute()->toString(),
          ],
        ]
      );

      $response = json_decode($response->getBody()->getContents());

      /** @var \Drupal\externalauth\ExternalAuth $externalauth */
      $externalauth = \Drupal::service('externalauth.externalauth');
      if ($this->config->get('register')) {
        $account_data['name'] = static::prepareUsername($response->me);
        $externalauth->loginRegister($response->me, static::PROVIDER, $account_data);
      }
      else {
        $externalauth->login($response->me, static::PROVIDER);
      }

      return new RedirectResponse(Url::fromRoute('<front>')->toString());
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'error' => 'invalid_request',
        'error_description' => $e->getMessage(),
      ], 400);
    }
  }

  /**
   * Prepare username from authname.
   *
   * @param string $authname
   *   The indieauth authname.
   *
   * @return string
   *   The reformatted username.
   */
  private static function prepareUsername(string $authname) {
    $parts = parse_url($authname);
    $path_parts = explode('/', $parts['path']);
    $string = implode('-', [$parts['host'], end($path_parts)]);
    return str_replace(['.', '/'], ['-', '-'], $string);
  }

}
